// Copyright (C) 2019  Wilko Manger
//
// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0. If a copy of the MPL was not distributed with this
// file, You can obtain one at http://mozilla.org/MPL/2.0/.

import 'package:matrix_sdk/src/event/room/room_event.dart';
import 'package:matrix_sdk/src/event/room/state/state_event.dart';
import 'package:meta/meta.dart';
import 'package:url/url.dart';

class RoomAvatarChangeEvent extends StateEvent {
  @override
  final RoomAvatarChange content;

  RoomAvatarChangeEvent(this.content, RoomEventArgs args)
      : super(args, stateKey: '');

  factory RoomAvatarChangeEvent.fromJson(RoomEventArgs args, dynamic json) {
    if (!hasStateKey(json)) {
      return null;
    }

    final url = json['content']['url'];

    // TODO: Handle avatar removals
    if (url == null) {
      return null;
    }

    return RoomAvatarChangeEvent(
      RoomAvatarChange(
        url: Url.parse(url),
      ),
      args,
    );
  }
}

class RoomAvatarChange extends StateEventContent {
  /// An `mxc` url pointing to the avatar.
  final Url url;

  RoomAvatarChange({@required this.url});

  @override
  Map<String, dynamic> toJson() => super.toJson()
    ..addAll({
      'url': url?.toString(),
    });
}
