// Copyright (C) 2019  Wilko Manger
//
// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0. If a copy of the MPL was not distributed with this
// file, You can obtain one at http://mozilla.org/MPL/2.0/.

import 'dart:convert';

import 'package:kiwi/kiwi.dart';
import 'package:matrix_sdk/src/api/client.dart';
import 'package:matrix_sdk/src/notifications/pusher.dart';

class Pushers {
  final ClientService _client;

  Pushers(String scope) : _client = Container().resolve<ClientService>(scope);

  Future<bool> _set(Map<String, dynamic> pusherJson) async {
    final response = await _client.setPusher(json.encode(pusherJson));

    switch (response.statusCode) {
      case 200:
        return true;
      default:
        return false;
    }
  }

  /// Set a pusher for this [LocalUser]. Returns true if successfully set.
  Future<bool> set(Pusher pusher) => _set(pusher.toJson());

  Future<bool> add(Pusher pusher) => _set(pusher.toJson()
    ..addAll({
      'append': true,
    }));

  /// Remove a pusher for this [LocalUser].
  /// Returns true if successfully removed.
  Future<bool> remove(Pusher pusher) {
    final json = pusher.toJson();
    json['kind'] = null;

    return _set(json);
  }
}
